package com.cultural.heritage.listener;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.entity.ClothesRentOrder;
import com.cultural.heritage.service.order.ClothesRentOrderService;
import com.cultural.heritage.utils.MultiDelayMessage;
import jakarta.annotation.Resource;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;


@Component
public class ClothesRentPeriodListener {


    @Resource
    private ClothesRentOrderService clothesRentOrderService;


    @Resource
    private RabbitTemplate rabbitTemplate;



    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(MqConstant.DELAY_CLOTHES_RENT_PERIOD_QUEUE),
            exchange = @Exchange(name = MqConstant.DELAY_EXCHANGE, delayed = "true"),
            key = MqConstant.DELAY_CLOTHES_RENT_PERIOD_ROUTING_KEY
    ))
    public void listenDelayMessage(MultiDelayMessage<Long> msg) {
        System.out.println("\n\n\n\n\nClothesRentPeriodListener.listenerDelayMessage msg-------------------------------->：" + msg);

        // 1.获取消息中的服装租赁订单id
        Long orderId = msg.getData();
        // 2.查询订单，判断状态是否为待收货
        ClothesRentOrder clothesRentOrder = clothesRentOrderService.getById(orderId);
        // 订单不存在或者订单状态不为待收货
        if (clothesRentOrder == null || !clothesRentOrder.getOrderStatus().equals(OrderStatusConstant.PENDING_DELIVERY)) {
            return ;
        }
        // 3.订单处于待收货状态，判断是否还有剩余延时时间
        if (msg.hasNextDelay()) {
            // 有延迟时间，需要重发延迟消息，先获取延迟时间的int值
            // 发送延时消息
            int delayValue = msg.removeNextDelay().intValue();
            rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                    MqConstant.DELAY_CLOTHES_RENT_PERIOD_ROUTING_KEY, msg, message -> {
                        // 添加延迟消息属性
                        message.getMessageProperties().setDelay(delayValue * 1000);
                        return message;
                    });
            return ;
        }
        // 没有剩余延时时间，说明订单交易完成，需修改订单状态
        UpdateWrapper<ClothesRentOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", orderId).set("orderStatus", OrderStatusConstant.TRANSACTION_SUCCESSFUL);
        boolean update = clothesRentOrderService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");
    }

}
