package com.cultural.heritage.service.address.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.AddressMapper;
import com.cultural.heritage.model.entity.Address;
import com.cultural.heritage.service.address.AddressService;
import com.cultural.heritage.utils.RegexUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
public class AddressServiceImpl extends ServiceImpl<AddressMapper, Address> implements AddressService {

    /**
     * 校验用户提交的地址信息
     */
    @Override
    public void validAddress(Address address, boolean update) {
        Long userId = address.getUserId();
        String name = address.getName();
        String phone = address.getPhone();
        String region = address.getRegion();
        String detailAddress = address.getDetailAddress();
        Integer isDefault = address.getIsDefault();

        if (update) {
            if (userId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数id错误");
            }
        }
        if (ObjectUtils.isEmpty(isDefault) || isDefault != 1 && isDefault != 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isAnyBlank(name, region, detailAddress, phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
        if (StringUtils.isNotBlank(phone) && RegexUtils.isPhoneInvalid(phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "手机号格式无效");
        }
    }

}
