package com.cultural.heritage.service.other.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.mapper.BannerMapper;
import com.cultural.heritage.model.dto.banner.BannerQueryRequest;
import com.cultural.heritage.model.entity.Banner;
import com.cultural.heritage.service.other.BannerService;
import com.cultural.heritage.utils.SqlUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
public class BannerServiceImpl extends ServiceImpl<BannerMapper, Banner> implements BannerService {


    /**
     * 校验
     */
    @Override
    public void validBanner(Banner banner, boolean update) {
        Long id = banner.getId();
        String type = banner.getType();
        String url = banner.getUrl();
        ThrowUtils.throwIf(update && id == null, ErrorCode.OPERATION_ERROR, "id参数错误");
        ThrowUtils.throwIf(StringUtils.isAnyBlank(type, url), ErrorCode.OPERATION_ERROR, "存在参数为空");
    }


    /**
     * 获取查询条件
     */
    @Override
    public QueryWrapper<Banner> getQueryWrapper(BannerQueryRequest bannerQueryRequest) {
        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        String type = bannerQueryRequest.getType();
        String sortField = bannerQueryRequest.getSortField();
        String sortOrder = bannerQueryRequest.getSortOrder();
        queryWrapper.eq(StringUtils.isNotBlank(type), "type", type);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC), sortField);
        return queryWrapper;
    }
}
